//----------------------------------------------------
//         EXPORT OF MOVEMENT BEAHAVIOUR (CAMERA ETC..)
//----------------------------------------------------

boolean forcedMovement = false;
int incrementOfPlace = 0, maxIncrementOfPlace = 9;
int curveMovement = 0;
float[][] oldPlace, newPlace; float oldXRotation, oldYRotation, oldZRotation, oldZoomFactor, newXRotation, newYRotation, newZRotation, newZoomFactor;
float[] oldZoomExtent = new float[3], newZoomExtent = new float[3];


void moveToNextPlace(){
  float t = bezierPoint(0,1,1,1,float(curveMovement)/60.0);
  float p = ((curveMovement/60.0)+2*t)/3.0;
  
  for(int i=0;i<newPlace.length;i++){
    elementInstances[i].myPos = interpolate(oldPlace[i],newPlace[i],t);
  }
  
  xRotation = lerp(oldXRotation,newXRotation,t);
  yRotation = lerp(oldYRotation,newYRotation,t);
  zRotation = lerp(oldZRotation,newZRotation,t);
  zoomExtent = interpolate(oldZoomExtent,newZoomExtent,p);
  zoomFactor = lerp(oldZoomFactor,newZoomFactor,p);
  
  curveMovement++;
  if(curveMovement>60)
    forcedMovement = false;
}


void saveMyPlace(){
  String[] myPlaceStrings = new String[elementInstances.length];
  for(int i = 0; i < elementInstances.length; i++) 
    myPlaceStrings[i] = "\""+elementInstances[i].myPos[0]+"\",\""+elementInstances[i].myPos[1]+"\",\""+elementInstances[i].myPos[2]+"\",\""+(elementInstances[i].hidden?1.0:0.0)+"\",\""+(elementInstances[i].pinned?1.0:0.0)+"\"";
  
  //SAVE POSITION OF CAMERA, SIZES, ETC.
  simplifyRotation();
  myPlaceStrings = splice(myPlaceStrings, "### NEW-AFTERDARK SETTINGS AND POSITIONS FILE V.1 ###", 0);
  myPlaceStrings = splice(myPlaceStrings, "CAMERA: "+xRotation+","+yRotation+","+zRotation+","+zoomExtent[0]+","+zoomExtent[1]+","+zoomExtent[2]+","+zoomFactor, 1);
  myPlaceStrings = splice(myPlaceStrings, "OPTIONS: "+visualInterface[0].myValue+","+visualInterface[1].myValue+","+visualInterface[2].inUse+","+visualInterface[2].myValue, 2);
  myPlaceStrings = splice(myPlaceStrings, "", 3);
  myPlaceStrings = splice(myPlaceStrings, "### POSITIONS ###", 4);
  
  saveStrings("data/showcase-AAA.txt",myPlaceStrings);
}


void loadNextToGoPlace(){
  String[] myPlaceStrings;
  float[][] myPlacePoints;

  myPlaceStrings = loadStrings("data/showcase-"+nf(incrementOfPlace,3)+".txt");
  
  //COPY THE OLD VALUES
  oldPlace = new float[myPlaceStrings.length][3];
  for(int i=0;i<elementInstances.length;i++)
    oldPlace[i] = elementInstances[i].myPos;
  simplifyRotation();
  oldXRotation = xRotation; oldYRotation = yRotation; oldZRotation = zRotation; oldZoomFactor = zoomFactor;
  arraycopy(zoomExtent,oldZoomExtent);
  
  //WITHDRAW THE NEW OPTION VALUES
  float[] newCam = float(splitTokens(myPlaceStrings[1],",: "));
  newXRotation = newCam[1]; newYRotation = newCam[2]; newZRotation = newCam[3]; 
  newZoomExtent[0] = newCam[4]; newZoomExtent[1] = newCam[5]; newZoomExtent[2] = newCam[6]; newZoomFactor = newCam[7];
  String[] newSettings = splitTokens(myPlaceStrings[2]);
  
  //WITHDRAW THE NEW POSITIONS
  myPlaceStrings = subset(myPlaceStrings, 5);
  newPlace = new float[myPlaceStrings.length][3];
  float[] rowMeaning = new float[5];
  for(int i=0;i<newPlace.length;i++){
    rowMeaning = float(splitTokens(myPlaceStrings[i],", \""));
    newPlace[i][0] = rowMeaning[0]; newPlace[i][1] = rowMeaning[1]; newPlace[i][2] = rowMeaning[2];
    elementInstances[i].hidden = (rowMeaning[3]>0.0); elementInstances[i].pinned = (rowMeaning[4]>0.0);
  }
  
  resetConnections();
  updateAllConnections();
  
  curveMovement = 0;
  forcedMovement = true;
}


//---------------------------------------------
//            FILE FORMATS EXPORTING
//---------------------------------------------

boolean export; int exportType=0; // 0=STL, 1=DXF, 2=COLLADA- NEEDS TO UNDO BEZIER CURVES BECAUSE NOT SUPORTED;
PrintWriter textOutput;

void startExport(){
     print("Beginning export.");
     showLeverage = false;
     
     
     if(exportType==0){ //RHINO COMMAND LINE
        
       textOutput = createWriter("D://rhino//data//export.rcl");
       showCurves = true;
        
       
       
     }else if(exportType==1){ //DXF
        try{
           beginRaw(DXF, "export"+frameCount+".dxf");
        }catch(Exception e){
          println(" Error creating file. Is it already existing?");
        }
        
        
       
      }else if(exportType==2){ //COLLADA
       try{
          //beginRecord(P5ColladaExport.DAE, "export"+frameCount+".dae");
       }catch(Exception e){
          println(" Error creating file. Is it already existing?");
       }
       drawText = false;
       showCurves = false;
       
       }else if(exportType==3){ //STL (UNLEKKER)
        try{
           beginRaw("unlekker.data.STL","export"+frameCount+".stl");
        }catch(Exception e){
          println(" Error creating file. Is it already existing?");
        }
          
          
        }else if(exportType==4){ //PDF
        try{
           beginRaw(PDF,"export"+frameCount+".pdf");
        }catch(Exception e){
          println(" Error creating file. Is it already existing?");
          }
        }
      
      
     
     textFont(font);
}


void endExport(){
  
    showCurves = true;
    drawText = true;
    showLeverage = true;
    export = false;
  
     if(exportType==0){
       textOutput.flush();
       textOutput.close();
     }else if(exportType==1)
       endRaw();
     else if(exportType==2)
       endRecord();
     else
       endRaw();
      
    exportType = -1;
    println(" Export is complete.");
}

//----------------------------------------
//        WRITING GEOMETRY
//----------------------------------------

void exportBezier(float x1, float y1, float z1, float cx1, float cy1, float cz1, float cx2, float cy2, float cz2, float x2, float y2, float z2){
  textOutput.println("_InterpCrv _S " +(0.0-x1)+","+z1+","+(0.0-y1)+ " " +(0.0-cx1)+","+cz1+","+(0.0-cy1)+" _E "+(0.0-cx2)+","+cz2+","+(0.0-cy2)+" "+(0.0-x2)+","+z2+","+(0.0-y2));
}

void exportLine(float x1, float y1, float z1, float x2, float y2, float z2){
  textOutput.println("_Line " +(0.0-x1)+","+z1+","+(0.0-y1)+ " "+(0.0-x2)+","+z2+","+(0.0-y2));
}

void exportBox(float x, float y, float z, float w, float l, float h){
  textOutput.println("_Box _Center " +(0.0-x)+","+z+","+(0.0-y)+ " "+(w*2.0)+" "+(h*2.0)+" "+(l*2.0));
}
