void simplifyRotation(){
     xRotation = xRotation%TWO_PI;
     yRotation = yRotation%TWO_PI;
     zRotation = zRotation%TWO_PI;
}

private float getDist(int a,int b){
  return sqrt ( sq(getDistX(a,b)) + sq(getDistY(a,b)) + sq(getDistZ(a,b)) );
}

private float[] getDistVector(int a,int b){
 float[] dist3D = {
    elementInstances[a].myPos[0]-elementInstances[b].myPos[0],
    elementInstances[a].myPos[1]-elementInstances[b].myPos[1],
    elementInstances[a].myPos[2]-elementInstances[b].myPos[2]  };
  return dist3D;
}

private float[] multiplyVector(float[] a,float b){
  float[] c = new float[3];
  c[0] = a[0]*b;
  c[1] = a[1]*b;
  c[2] = a[2]*b;
  return c;
}

private float[] addVector(float[] a,float[] b){
  float[] c = new float[3];
  c[0] = a[0]+b[0];
  c[1] = a[1]+b[1];
  c[2] = a[2]+b[2];
  return c;
}

private float[] subtractVector(float[] a,float[] b){
  float[] c = new float[3];
  c[0] = a[0]-b[0];
  c[1] = a[1]-b[1];
  c[2] = a[2]-b[2];
  return c;
}

private float getDistX(int a,int b){
  return elementInstances[a].myPos[0]-elementInstances[b].myPos[0];
}

private float getDistY(int a,int b){
  return elementInstances[a].myPos[1]-elementInstances[b].myPos[1];
}

private float getDistZ(int a,int b){
  return elementInstances[a].myPos[2]-elementInstances[b].myPos[2];
}

private float getRadXSum(int a,int b){
  return elementInstances[a].mySize[0]+elementInstances[b].mySize[0];
}

private float getRadYSum(int a,int b){
  return elementInstances[a].mySize[1]+elementInstances[b].mySize[1];
}

private float getRadZSum(int a,int b){
  return elementInstances[a].mySize[2]+elementInstances[b].mySize[2];
}

private float sign(float a){
  if(a==0.0){//println("Error: the float inside sign() is 0."); 
  return 1.0; }
  return (a)/abs(a);
}

private float[] normalize(float[] v){
   float tempd=sqrt (sq (v[0]) +sq(v[1]) + sq(v[2]));
   if(tempd==0.0){ println("Error: the vector inside normalize() is 0."); return v; };
   v[0] = v[0]/tempd;
   v[1] = v[1]/tempd;
   v[2] = v[2]/tempd;
   return v;
}

private float[] interpolate(float[] v1,float[] v2, float c){
 float[] interp3D = {
  v1[0]*(1.0-c)+v2[0]*c,
  v1[1]*(1.0-c)+v2[1]*c,
  v1[2]*(1.0-c)+v2[2]*c
  };
  return interp3D;
}

private void resetConnections(){
   for(int i=0;i<elementInstances.length;i++){
     elementInstances[i].readMyConnectionText();
   }
}

private void updateAllVisibility(){
    for(int i=0;i<elementInstances.length;i++){
         elementInstances[i].updateVisibility();
         }
}

private void updateAllConnections(){
  //Goes throught the whole tree and skips the hidden connections
  
  for(int i=0;i<elementInstances.length;i++){
    
      for(int h=0; h<elementInstances[i].myConnections.length; h++){
          if(elementInstances[elementInstances[i].myConnections[h]].hidden){
             elementInstances[i].myConnections = concat(elementInstances[i].myConnections, elementInstances[elementInstances[i].myConnections[h]].myConnections); //Adds the rest 
             elementInstances[i].myConnections = concat(subset(elementInstances[i].myConnections,0,h),subset(elementInstances[i].myConnections,h+1)); //Deletes the one we dont want
             elementInstances[i].myConnections = sort(elementInstances[i].myConnections);
             elementInstances[i].myConnections = makeListUnique(elementInstances[i].myConnections);
             h=0;
          }
         }
       }
    
}

private int[] makeListUnique(int[] myList){ 
  for(int i=1;i<myList.length;i++){
   if(myList[i]==myList[i-1])
    myList = concat(subset(myList,0,i),subset(myList,i+1));
  }
return myList;
}

private void initializeAllVelocities(){
   for(int i=0;i<elementInstances.length;i++){
     if(!elementInstances[i].pinned && !elementInstances[i].hidden)
       elementInstances[i].initializeVelocity();
   }
}

private void stopAllVelocities(){
   for(int i=0;i<elementInstances.length;i++){
       elementInstances[i].stopVelocity();
   }
}

private void mirrorScene(){
   for(int i=0;i<elementInstances.length;i++){
       elementInstances[i].myPos[0]=elementInstances[i].myPos[0]*-1.0;
       elementInstances[i].myVel[0]=elementInstances[i].myVel[0]*-1.0;
   }
}

private float[] absVector(float[] v){
  float c[] = new float[3];
  c[0] = abs(v[0]);
  c[1] = abs(v[1]);
  c[2] = abs(v[2]);
 return c;
}

private float vectorMagnitude(float[] a){
 return sqrt(sq(a[0])+sq(a[1])+sq(a[2]));
}


private float dotProduct(float[] a, float[] b){
  return a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
}

private float giveAngle(float[] a, float[] b){
  // a.b (dotproduct) == |a||b| cos t
  float t = dotProduct(a,b)/(vectorMagnitude(a)*vectorMagnitude(b));
  return acos(t);

}

private float[] findImpactNormal(float[] a, float[] b, float[] wa, float[] wb){
  float[] d = subtractVector(a,b);
  float[] w = addVector(wa,wb);
  float[] v = subtractVector(w,absVector(d));
  
  float[] T = {0,0,0};
  
 if(v[0]<v[1] && v[0]<v[2]){
   if(d[0]>0)
    T[0] = d[0]-w[0];
   else
    T[0] = d[0]+w[0];
 }else if(v[1]<v[2]){
   if(d[1]>0)
    T[1] = d[1]-w[1];
   else
    T[1] = d[1]+w[1];
 }else{
   if(d[2]>0)
    T[2] = d[2]-w[2];
   else
    T[2] = d[2]+w[2];
 }
 return T;
}

private void avoidObjectsCrossing(){
  
   for(int i=0;i<elementInstances.length-1;i++){
      for(int j=i+1;j<elementInstances.length;j++){
        
        
        if(!elementInstances[i].hidden && !elementInstances[j].hidden){
          
        if(cuboid_collision(i,j)){
         
          // get normal
	  float[] n = findImpactNormal(elementInstances[i].myPos,elementInstances[j].myPos, elementInstances[i].mySize, elementInstances[j].mySize);
          
          
          //set (OUTSIDE OF EACHOTHER) to new position. Uses an approximated repositioning on the main axis
          if(n[0] != 0.0){
            if(!elementInstances[i].pinned)
            elementInstances[i].myPos[0] = elementInstances[i].myPos[0]-n[0]/2.0;
            if(!elementInstances[j].pinned)
            elementInstances[j].myPos[0] = elementInstances[j].myPos[0]+n[0]/2.0;
            n[0] = sign(n[0]);
          } else if(n[1] != 0.0){
            if(!elementInstances[i].pinned)
            elementInstances[i].myPos[1] = elementInstances[i].myPos[1]-n[1]/2.0;
            if(!elementInstances[j].pinned)
            elementInstances[j].myPos[1] = elementInstances[j].myPos[1]+n[1]/2.0;
            n[1] = sign(n[1]);
          }else{
            if(threeD){
            if(!elementInstances[i].pinned)
            elementInstances[i].myPos[2] = elementInstances[i].myPos[2]-n[2]/2.0;
            if(!elementInstances[j].pinned)
            elementInstances[j].myPos[2] = elementInstances[j].myPos[2]+n[2]/2.0;
            n[2] = sign(n[2]);
            }
          }
          
          
          
	  // dotProduct * factor in elasticity
          float vDotN1 = dotProduct(n,elementInstances[i].myVel) * 1.8;
	  float vDotN2 = dotProduct(n,elementInstances[j].myVel) * 1.8;
					
	  // reflect veloctiy along normal
          float[] oldVel = {0,0,0};
          arraycopy(elementInstances[i].myVel,oldVel);
	  elementInstances[i].setVel(elementInstances[i].myVel[0]-vDotN1*n[0],elementInstances[i].myVel[1]-vDotN1*n[1], elementInstances[i].myVel[2]-vDotN1*n[2]);
          // this part is only necessary to avoid endless little rebouncing in case the wanted distance is inside the other cuboid - if you disable it lower elasticity
          elementInstances[i].myVel = interpolate(oldVel,elementInstances[i].myVel,0.72);
          
          arraycopy(elementInstances[j].myVel,oldVel);
	  elementInstances[j].setVel(elementInstances[j].myVel[0]-vDotN2*n[0],elementInstances[j].myVel[1]-vDotN2*n[1], elementInstances[j].myVel[2]-vDotN2*n[2]);
          // this part is only necessary to avoid endless little rebouncing in case the wanted distance is inside the other cuboid - if you disable it lower elasticity
          elementInstances[j].myVel = interpolate(oldVel,elementInstances[j].myVel,0.72);
        }
          
          
        }}
   }
}

float linear_intersect(float x_1, float width_1, float x_2, float width_2){
  
  float d=x_1-x_2; float s=width_1+width_2;
    if( d > s || d < -2*s)
     return 0;
    
    if(d>0)
      return d-s;
    else
      return d+s;
}

//THANKS TO PROCESSINGHACKS.ORG
boolean line_collision(float x_1, float width_1, float x_2, float width_2){
  return !(x_1 > x_2+width_2 || x_1+width_1 < x_2);
}

boolean rect_collision(float x_1, float y_1, float width_1, float height_1, float x_2, float y_2, float width_2, float height_2){
  return !(x_1 > x_2+width_2 || x_1+width_1 < x_2 || y_1 > y_2+height_2 || y_1+height_1 < y_2);
}

boolean cuboid_collision(int a,int b){
  if(line_collision(elementInstances[a].myPos[0], elementInstances[a].mySize[0]*2, elementInstances[b].myPos[0], elementInstances[b].mySize[0]*2)){
    if(line_collision(elementInstances[a].myPos[1], elementInstances[a].mySize[1]*2, elementInstances[b].myPos[1], elementInstances[b].mySize[1]*2)){
      return line_collision(elementInstances[a].myPos[2], elementInstances[a].mySize[2]*2, elementInstances[b].myPos[2], elementInstances[b].mySize[2]*2);
    }else return false;
  }else return false;
}

