/////////////////////////////////////////////////////////////////////////////////////
//  COPYRIGHT GIULIO PIACENTINO 2008. SOME RIGHTS RESERVED.                        //
//  DEVELOPED PARTIALLY IN THE HYPERBODY DESIGN GROUP OF TU DELFT, UP TO JULY 2008 //
//  MENTORED BY: AXEL KILIAN, KAS OOSTERHUIS, NIMISH BILORIA AND  HENRIETTE BIER   //
//                                                                                 //
//  THIS SOFTWARE IS PROVIDED TO YOU "AS IS", WITHOUT ANY WARRANTY                 //
//  IT IS LICENCED TO YOU AT THE CONDITIONS SHOWN ON THE WEBPAGE:                  //
//  http://www.giuliopiacentino.com/this/                                          //
//                                                                                 //
// PLEASE, IF YOU USE IT AND LIKE IT, DROP A MAIL AT: mail-at-giuliopiacentino-com //
// SEE THE END OF THIS FILE FOR A SMALL INTRO TO HOW IT WORKS                      //
/////////////////////////////////////////////////////////////////////////////////////

import processing.opengl.*;

//general variables
color myBack, myCol, myStr, mySel, myPin, myTxt, myFogC, oroFront, oroStroke, oroSel;
boolean repaint = true, executing = false, threeD = false, fastMode = true, showLeverage = false, showCurves = false, drawText = true;
boolean checkBorders = false, checkObjectsEdges = false, viewPerspective = true, autoRotate = false, weatherOn = false, lightsOn = false, addFog = false;
int drawOrography = 0;

float drawingScale = 1.0/(   1.0/4.0  ); //X pixels in drawing are corresponding to X meters in reality (if Z==0). The dimensions are calculated in meters, the drawing is in pixels

String[][] diagram = {
  //   Name, ConnectedTo, ImportanceOfThisNode, SizeFormula(m^2 = A+[B/C* (hl) ]^(D/E)), lightNeed [0-1]
  
  {  "field\ndelivery", "-1", "1.0", "50,1,10", "1.0"},      //ID: 0
  {  "reception", "0", "0.5", "1", "0.9"},                   //ID: 1
  {  "crushing", "0", "0.3", "10,1,2,1,3" , "0.1"},          //ID: 2
  {  "office\ngrape\ntestings", "0", "0.6", "12", "0.8"},    //ID: 3
  {  "fermenta-\ntion", "2", "0.8", "5,1,10" , "0.1"},       //ID: 4
  {  "press", "4", "0.3", "8,1,3,1,2", "0.1"},               //ID: 5
  {  "laboratory", "3,4", "0.6", "5" , "0.6"},               //ID: 6
  {  "ageing\nin\nsteel", "5", "0.8", "20,1,10", "0.4"},     //ID: 7
  {  "ageing\nin\noak", "4,5", "0.8", "10,1,9", "0.2"},      //ID: 8
  {  "bottling", "7,8", "0.8", "6", "0.1"},                  //ID: 9
  {  "tasting\nmaturation","8,9","0.7","14,1,10,1,2", "0.8"},//ID: 10
  {  "capsules\nlabels", "9,10", "0.3", "8", "0.1"},         //ID: 11
  {  "bottle\nstorage", "11", "0.3", "20,1,50", "0.1"},      //ID: 12
  {  "final\npackaging", "10,12", "0.4", "20" , "0.1"},      //ID: 13
  {  "sales\narea", "10,13", "0.5", "14,1,10,1,2" , "0.8"},  //ID: 14
  {  "bar", "14", "0.6", "10,1,60", "0.9"},                  //ID: 15
  {  "visitor's\ntoilets", "14", "0.3", "12", "0.2"},        //ID: 16
  {  "workers's\narea", "9,16", "0.4", "12" , "0.2"},        //ID: 17
  {  "transpor\ntation", "14", "0.2", "30,1,15", "0.5"},     //ID: 18
  {  "courtyard\nshipping", "18", "0.9", "50,1,10", "1.0"}   //ID: 19
};


myLeverage[] visualInterface = new myLeverage[3];
TextField textField = new TextField(false);

myBox[] elementInstances = new myBox[diagram.length];
String explainedSelection = "", addInfo = "";
float xRotation, yRotation, zRotation, zoomFactor; 
float[] zoomExtent = { 0.0,0.0,0.0};
PFont font;



void setup (){
  
  size (screen.width,screen.height-20,OPENGL);
  //size (800,600,OPENGL);

  ellipseMode (CENTER);
  rectMode(CENTER);
  textureMode(NORMALIZED);
  
  myBack = color(201);
  myCol = color(240, 240, 240);
  myStr = color(255, 255, 255);
  mySel = color(246,255,53);
  myPin = color(255,164,2);
  myTxt = color(0);
  myFogC = myBack;
  oroFront = color(0,35,99,55);
  oroStroke = #688855;
  oroSel = #990000;
  
  font = loadFont("OCRAExtended-10.vlw"); 
  textFont(font);
  hint(ENABLE_NATIVE_FONTS);
  textAlign(CENTER,CENTER);

  zoomFactor = 1.0;
  xRotation = 0;
  yRotation = 0;
  zRotation = 0;
  

  //num, name, startvalue, m, c, approximation  ----  (m,c as in   y = mx + c)
  visualInterface[0] = new myLeverage(0, "c.ty", 0.4, -1.0, 1.0, 0.01, true);  //THE MINIMAL VALUE TO SHOW CONNECTIONS
  visualInterface[1] = new myLeverage(1, "d", 12.0, 30.0, 0.0, 0.1, true);    //THE ADDITIONAL DISTANCE TO OBTAIN
  visualInterface[2] = new myLeverage(2, "hl", 500.0, 9900.0, 100.0, 10.0, false);    //THE INPUT OF GRAPES
  
  
  
  float[] myAcc, myPos;
  for (int i = 0; i < diagram.length; i = i+1){

    myPos = new float[3];
    myAcc = new float[3];

    myPos[0] = random(-width/3.0,width/3.0);
    myPos[1] = (height*(i+1.0)/(diagram.length+1.0))-height/2.0;
    myPos[2] = random(-5.0,5.0);

    //    Radius, StartingVelocity, Direction (3 Arr), Position (3 Arr), Color, Stroke (if -1 -> no stroke), settings, identity.
    elementInstances [i]= new myBox (myPos, myCol, myStr, diagram[i], i);
    //elementInstances[i].initializeVelocity();

    //Pin first and last entry
    if(i == 0 || i==diagram.length-1)
      elementInstances[i].pinned = true;
  }


  //Update all connections for hidden entries
  updateAllVisibility();
  updateAllConnections();


  //SETTINGS FOR OROGRAPHY
  importScale = 1;
  tolerance = 0.1;
  orographyDepth = 0; 
  orographyWidth = 0;  //If both are == 0 the program tries to guess which one is the right size of the rectangle, using the parameter tolerance
  orography = loadMountainFile();


  //THE ONLY EASY WAY I FOUND TO GET THE CENTRAL WHEEL WORK
  frame.addMouseWheelListener(new MouseWheelInput());
  
}







void draw (){
  

  //DRAW THE BACKGOUND
  if(repaint)
    background (myBack);

    if(export)
      startExport();

    if(autoRotate)
      yRotation=yRotation+TWO_PI/1200.0;

    pushMatrix();
    translate(width/2+zoomExtent[0],height/2+zoomExtent[1],zoomExtent[2]);
    rotateX(xRotation);
    rotateY(yRotation);
    rotateZ(zRotation);
    scale(zoomFactor);
    

    if(forcedMovement){
      //MOVE SMOOTHLY TO THE REGISTERED POSITION
      moveToNextPlace();
    }
    else{
      //UPDATE THE FORCES
      if(executing){
        for (int i = 0; i < elementInstances.length; i++)
          elementInstances[i].findForces();

        for (int i = 0; i < elementInstances.length; i++)
          elementInstances[i].updatePosition();
      }        
    }

    //AVOID PENETRATION OF OBJECTS
    if(checkObjectsEdges)
      avoidObjectsCrossing();

    //DRAW THE LINES
    strokeWeight(2);
    stroke(myStr);
    noFill();

    if(showCurves)
      for (int i = 0; i < elementInstances.length; i++)
        elementInstances[i].drawLines();

    for (int i = 0; i < elementInstances.length; i++)
      elementInstances[i].updateScreenPos();

    //DRAW THE BOXES
    strokeWeight(2);
    fill(myCol);
    for (int i = 0; i < elementInstances.length; i++) {
      elementInstances[i].drawBox();
    }

    //THINKGS FOR THE SELECTED ENTRY
    if(selIndex>-1)
      elementInstances[selIndex].thingsWhenSelected();

    //DRAW OROGRAPHY
    if(drawOrography == 0){


    } 
    else if(drawOrography == 1){

      for(int i=0; i<orographyDepth-1; i++){
        fill(oroFront);
        beginShape(QUAD_STRIP);
        for(int j=0; j<orographyWidth; j++){
          vertex(orography[i][j][0], orography[i][j][1], orography[i][j][2]);
          vertex(orography[i+1][j][0], orography[i+1][j][1], orography[i+1][j][2]);
        }
        endShape();
      }

    } 
    else if(drawOrography == 2){

      for(int i=0; i<orographyDepth-1; i++){
        noStroke();
        fill(oroFront);
        beginShape(QUAD_STRIP);
        for(int j=0; j<orographyWidth; j++){
          vertex(orography[i][j][0], orography[i][j][1]+1.5, orography[i][j][2]);
          vertex(orography[i+1][j][0], orography[i+1][j][1]+1.5, orography[i+1][j][2]);
        }
        endShape();
      }
      stroke(oroStroke);
      noFill();
      for(int i=0; i<orographyDepth-1; i++){
        beginShape();
        for(int j=0; j<orographyWidth; j++){
          curveVertex(orography[i][j][0], orography[i][j][1], orography[i][j][2]);
          curveVertex(orography[i+1][j][0], orography[i+1][j][1], orography[i+1][j][2]);
        }
        endShape();
      }

    } 
    else if(drawOrography == 3){

      for(int i=0; i<orographyDepth-1; i++){
        beginShape();
        for(int j=0; j<orographyWidth; j++){
          curveVertex(orography[i][j][0], orography[i][j][1], orography[i][j][2]);
          curveVertex(orography[i+1][j][0], orography[i+1][j][1], orography[i+1][j][2]);
        }
        endShape();
      }
    }
    
    
    


    popMatrix();

    hint(DISABLE_DEPTH_TEST);

    fill(myTxt);
    if(drawText){
      for (int i = 0; i < elementInstances.length; i++)
        elementInstances[i].addTxt();
    }



  if(showLeverage){

    //DRAW THE INTERFACE
    stroke(myStr);
    strokeWeight(1.0);

    if(selIndex<-1){
       fill(myBack);
       rect(width-30,height-(0-selIndex-2)*90-36,30,70);
    }

    fill(myPin);

    for (int i = 0; i < visualInterface.length; i++)
      visualInterface[i].drawInterface();

    //DRAW SOME INFORMATIVE TEXT
    fill(myTxt);
    if(selIndex>-1){
      explainedSelection = ""+selIndex;
      elementInstances[selIndex].getBoxInfo();
    }
    else{
      if(selIndex==-1)
        explainedSelection = "nothing";
      else
        explainedSelection = visualInterface[0-selIndex-2].myProp;
    }
    text("F: "+int(frameRate)+", s: "+explainedSelection+addInfo,60,height-12);
    //text("0-9 select, Before, Next; Pin, miX, Stop, Go; Hide!, Execution!, Repaint!, Fastmode!", width/2 ,height-12);
    for(int i = 0; i < visualInterface.length; i++)
      visualInterface[i].drawInterfaceTxt();
  }

  hint(ENABLE_DEPTH_TEST);

  if(export)
    endExport();
  
  //saveFrame("data/"+nf(frameCount,5)+".jpg");
}


/*
This release has been tested in Processing 1.0.1 and, previously,
it is known to work in every release of Processing from 135 except 148-149.

At the very beginning of this file you can set up the names of the rooms of your swarming building.

They also have some inherited settings that you can try tweaking to achieve your results.
Java does not ship with a math formula interpreter, so there is a simple math formula to translate
an equation into its result.
There is a lot of code that has been commented out or deleted. It was using libraries that I felt
were not completely necessary any longer.

The setup works like this:
at the beginning of the sketch, the setup() creates all the Java objects (there are comments inside).
Then, for each draw(), there are some functions that move objects (one by one) and do the anti-colliding
if you specify so.
These function themselves are inside the tab "boxes".

The rest of the tabs:
- "exporting" takes care of many types of import of scenarios / export of data for Rhino
- "geometry" is some sort of functions library with queries for geometrical and behavioural stuff
- "graphics" does little at the moment
- "leverages" is the tab that contains drawing and behavioural methods for the scrollbars on the right
  bottom of the sketch
- "orography" takes care of opening txt files of points that you can easily, for example, export from Rhino,
  it puts them in order (on the fly, if you want it to save the result just turn saveMyNewFile = true).
  It uses an Odd-Even Transposition sort algorithm
- "TextListener" adds the possibility of inserting text into the leverages (it is another class). Just select
  one leverage and write a number, then press enter
- "zKeyInterface" can be used to activetes commands
- "zMouseInterface" contains the mouse actions for selecting, moving, etc..


*/
